//=====================================================================================
#include "gheader.h"
//=====================================================================================
#ifdef ENABLE_FLASH_MODE 
//=====================================================================================
/*
϶@
Flash Address Range : 0x3000 ~ 0x33FF
Flash size : 1024
EEPROM BYTE : 64
Group size (FlashIndex): 1024 / 64 = 16
Write Total Time : 16 * 1000 = 16K
*/

//The WT51F516 flash last 4K can simulator EEPROM
//The WT51F516 total flash is 16K Bytes 0x0000 ~ 0x3FFF
//Simulator EEPROM Range :  0x3000 ~ 0x3FFF
/*
Using Flash function, please follow this step:
1. Eeprom size = FLASH_DEFAULT_SIZE
2. define flash start address = FLASH_DEFAULT_ADDRESS
3. define flash size = FLASH_SIZE
4. modify erase area = DRV_EraseFlash(x)
*/




BYTE idata FlashData[FLASH_DEFAULT_SIZE];	 		

BYTE FlashIndex;													 

BYTE FlashAddressLow;
BYTE FlashAddressHigh;

//=====================================================================================
//
//=====================================================================================
//For Erase function address

#define Bank0AddressLow			Bank0ADDRESS%256			
#define Bank0AddressHigh		Bank0ADDRESS>>8						

#define Bank1AddressLow			Bank1ADDRESS%256				
#define Bank1AddressHigh		Bank1ADDRESS>>8						

#define Bank2AddressLow			Bank2ADDRESS%256				
#define Bank2AddressHigh		Bank2ADDRESS>>8						

#define Bank3AddressLow			Bank3ADDRESS%256				
#define Bank3AddressHigh		Bank3ADDRESS>>8						

#define Bank4AddressLow			Bank4ADDRESS%256				
#define Bank4AddressHigh		Bank4ADDRESS>>8				

#define Bank5AddressLow			Bank5ADDRESS%256				
#define Bank5AddressHigh		Bank5ADDRESS>>8				

#define Bank6AddressLow			Bank6ADDRESS%256				
#define Bank6AddressHigh		Bank6ADDRESS>>8				

#define Bank7AddressLow			Bank7ADDRESS%256				
#define Bank7AddressHigh		Bank7ADDRESS>>8				





//==============================================================
//Bank Erase  1 Bank = 512 bytes
//Erase 512 bytes time= 10ms
//Erase 4K bytes time= 10ms*8=80ms
//==============================================================
void DRV_EraseFlash(BYTE bank)
{
	rEER_EN1=0x0A; 											//;EEPROM enable data1: 0x0A
	rEER_EN2=0x05; 											//EEPROM enable data2: 0x05
	switch(bank)
	{	
		case 0:
			//set erase EEPROM 0x3000~0x31FF
			rEER_ADDRL = Bank0AddressLow;
			rEER_ADDRH = Bank0AddressHigh;										
			break;
		case 1:
			//set erase EEPROM 0x3200~0x33FF
			rEER_ADDRL = Bank1AddressLow;
			rEER_ADDRH = Bank1AddressHigh;										
			break;
		case 2:
			//set erase EEPROM 0x3400~0x35FF
			rEER_ADDRL = Bank2AddressLow;
			rEER_ADDRH = Bank2AddressHigh;										
			break;
		case 3:
			//set erase EEPROM 0x3600~0x37FF
			rEER_ADDRL = Bank3AddressLow;
			rEER_ADDRH = Bank3AddressHigh;										
			break;
		case 4:
			//set erase EEPROM 0x3800~0x39FF
			rEER_ADDRL = Bank4AddressLow;
			rEER_ADDRH = Bank4AddressHigh;										
			break;
		case 5:
			//set erase EEPROM 0x3A00~0x3BFF
			rEER_ADDRL = Bank5AddressLow;
			rEER_ADDRH = Bank5AddressHigh;										
			break;
		case 6:
			//set erase EEPROM 0x3C00~0x3DFF
			rEER_ADDRL = Bank6AddressLow;
			rEER_ADDRH = Bank6AddressHigh;										
			break;
		case 7:
			//set erase EEPROM 0x3E00~0x3FFF
			rEER_ADDRL = Bank7AddressLow;
			rEER_ADDRH = Bank7AddressHigh;										
			break;

		default:
			//set erase EEPROM 0x3000~0x31FF
			rEER_ADDRL = Bank0AddressLow;
			rEER_ADDRH = Bank0AddressHigh;										
			break;		
	}
	rEER_ERASE = ENABLE_ERASE + ERASE_TIMING;;
	rEER_EN1 = 0x00; 											//;EEPROM disable data1: 0x00
	rEER_EN2 = 0x00; 											//EEPROM disable data2: 0x00
}


//===============================================
//FlashIndex is 1 BYTE 
//Unknow write count, scan flash then find to start save address
//===============================================
WORD GetFlashAddress(void)
{
	WORD 	R6;
	WORD	wFlashAddress;	
	for(R6=0;R6<FLASH_GROUP_SIZE;R6++)
	{
		wFlashAddress = FLASH_DEFAULT_ADDRESS + FLASH_DEFAULT_SIZE*R6;		 				
	 	if(BASE_ADDR[wFlashAddress] > FLASH_GROUP_SIZE)
		{
			wFlashAddress = wFlashAddress - FLASH_DEFAULT_SIZE;
		 	return wFlashAddress;		
		}			
	}
	return wFlashAddress;	
}
//===============================================
//EepromData write to Flash.
//Total 256 bytes 
//program time=256/8*40us=1.28ms
//===============================================
void DRV_SaveFlashData(void)
{
	BYTE R5;
	WORD wWriteAddress,wNextAddress;
	BYTE bFlashCorrectStatus=0x55;
	BYTE bFlashErrorCount=0;
	while(bFlashCorrectStatus==0x55)
	{
	
		//Reset Watchdog
		OS_Reset_WD();
		if(bFlashErrorCount > FLASH_GROUP_SIZE)
		{
			return;
		}
		//Eeprom Index
		FlashIndex = FlashData[0] + 1;
		if(FlashIndex > FLASH_GROUP_SIZE)
 	    {
		FlashIndex = FLASH_GROUP_SIZE + 1;
		}
		FlashData[0] = FlashIndex; 		
		//DRV_Printf("FlashIndex = %x \r\n",FlashIndex);
		wWriteAddress = FLASH_DEFAULT_ADDRESS + ((FlashIndex-1) * FLASH_DEFAULT_SIZE);
		//DRV_Printf("FLASH_OVER_ADDRESS = %x \r\n",FLASH_OVER_ADDRESS);
		//DRV_Printf("wWriteAddress = %x \r\n",wWriteAddress);
		//Check flash limit address
		if(wWriteAddress >= FLASH_OVER_ADDRESS)
		{
			//DRV_EraseFlash(0);
			//DRV_EraseFlash(1);
			//DRV_EraseFlash(2);
			//DRV_EraseFlash(3);
			//DRV_EraseFlash(4);
			//DRV_EraseFlash(5);
			//DRV_EraseFlash(6);
			DRV_Printf("Erase Flash  %x \r\n",0);	
			DRV_EraseFlash(7);
			wWriteAddress = Bank7ADDRESS;
			FlashIndex = 1;
			FlashData[0] = 1;			
		}

		#if(DEBUG_FLASH==ON)
		//DRV_Printf("Write Start Flash Address = %x \r\n",wWriteAddress);				
		#endif

//========================================
		//Start write data to flash
		for(R5=0;R5<FLASH_WRITECYCLE;R5++)
		{
			//set write address
			wNextAddress = wWriteAddress + R5;

			FlashAddressLow = wNextAddress;
			FlashAddressHigh = (wNextAddress>>8) & 0x0F;

			rEER_EN1 = 0x0A; 								//EEPROM enable data1: 0x0A
			rEER_EN2 = 0x05; 								//EEPROM enable data2: 0x05
			rEER_ADDRL = FlashAddressLow;					//set EEPROM address 	
			rEER_ADDRH = FlashAddressHigh;

			rEER_DATA0 = FlashData[R5];
			

			rEER_ERASE = (ENABLE_PROGRAM | PROGRAM_TIMING);
		
			rEER_EN1=0x00; 																//;EEPROM disable data1: 0x00
			rEER_EN2=0x00; 																//EEPROM disable data2: 0x00

		}

		#if(DEBUG_FLASH==ON)
			DRV_Printf(" Now Write Flash Address = %x \r\n ",wWriteAddress);								
		#endif
		

 		for(R5=0;R5<FLASH_DEFAULT_SIZE;R5++)
		{
			if(BASE_ADDR[wWriteAddress+R5] != FlashData[R5])
		  	{
				#if(DEBUG_FLASH==ON)
				DRV_PutStr(" Flash Write Fail");								
				#endif
				//fail
		  		bFlashCorrectStatus = 0x55;
				
				return;

		  		break;
		  	}
			else
			{
				//correct
				bFlashCorrectStatus = 0x00;	
			}			
		}
	}
}


//=========================================================
//
//=========================================================
void DRV_ReadFlashData(void)
{
	BYTE 	R5=0;
	WORD	wReadAddress;

	wReadAddress = GetFlashAddress();

	#if(DEBUG_FLASH==ON)
	DRV_Printf("Read	Flash Address = %x \r\n",wReadAddress+R5);	
	#endif

	for(R5=0;R5<FLASH_DEFAULT_SIZE;R5++)
	{	
		FlashData[R5] = BASE_ADDR[wReadAddress+R5];	
	}

 	#if(DEBUG_FLASH==ON)	
	DRV_Printf("  Read	Flash Data = %d \r\n",FlashData[0]);
		DRV_Printf("FlashData[0] = %d \r\n",FlashData[0]);
		DRV_Printf("FlashData[1] = %d \r\n",FlashData[1]);
		DRV_Printf("FlashData[2] = %d \r\n",FlashData[2]);
		DRV_Printf("FlashData[3] = %d \r\n",FlashData[3]);
		DRV_Printf("FlashData[4] = %d \r\n",FlashData[4]);
		DRV_Printf("FlashData[5] = %d \r\n",FlashData[5]);
		DRV_Printf("FlashData[6] = %d \r\n",FlashData[6]);
		DRV_Printf("FlashData[7] = %d \r\n",FlashData[7]);	 	
	#endif	
	
}
#endif
