#ifndef DRV_IR_H
#define DRV_IR_H
//IR clock ratio



//=====================================================================================
//
//=====================================================================================
enum
{
  //H/W IR edge trigger
  IR_BOTH_EDGE = 0,
  IR_SINGLE_EDGE,
  //H/W IR edge trigger type
  IR_EDGE_FALLING = 0,
  IR_EDGE_RISING
};
//=====================================================================================
//
//=====================================================================================
#if (SYS_XTAL == XTAL_11M)
	#define IR_CLK_RATIO         0.9216	 // 11.0592/12
#elif (SYS_XTAL == XTAL_12M)
	#define IR_CLK_RATIO         1	 // 12/12
#elif (SYS_XTAL == XTAL_22M)
	#define IR_CLK_RATIO         0.9216	// 22.1184/2/12	
#elif (SYS_XTAL == XTAL_24M)
	#define IR_CLK_RATIO         1	// 24/2/12
#endif




//Pin define
#define GPIO_IR_IO_FUN_EN         DefineBit(rGPIOA_OEN, bit1)
#define GPIO_IR    				  DefineBit(rGPIOA_DAT, bit1)

#define HW_IR_FUN_EN              DefineBit(rIR_CTL, bit7)
#define HW_IR_CTL_CLR_INT         DefineBit(rIR_CTL, bit0)   //Clear H/W IR interrupt

#define HW_IR_STA_HL              DefineBit(rIR_STA, bit2)   //H/W IR input H/L
#define HW_IR_STA_OVFLW           DefineBit(rIR_STA, bit1)   //H/W IR overflow interrupt






//IR Option Setup 		
#define ACTIVE_HW_IR            ON
#define IR_POLL_TIME            1   //Base on 10ms





#define HW_IR_CNT_RANGE         0.2    // Range +-20%

  
	
//IR Key define
#define IR_KEY_NULL			0x00


//Detect key(for power down)
#define ir_detectkey0         PMCTL_DATA[PMCTL_DATA_IR_CODE0_NO]
#define ir_detectkey1         PMCTL_DATA[PMCTL_DATA_IR_CODE1_NO]
#define ir_detectkey2         PMCTL_DATA[PMCTL_DATA_IR_CODE2_NO]
#define ir_detectkey3         PMCTL_DATA[PMCTL_DATA_IR_CODE3_NO]



#if(HW_IR_DATA_TYPE == NEC_TYPE)
	#define HW_IR_SEDG                  IR_SINGLE_EDGE
	#define HW_IR_EDGE_TYPE        IR_EDGE_FALLING
	#define HW_IR_OV_INT              ON	
	#define HW_IR_PRE_SCAL          IR_PRE_SCAL_64us				  
	//#define HW_IR_PRE_SCAL          IR_PRE_SCAL_128us				//for 24Mhz for WT61P7
	#define IR_Count 					NEC_TYPE1
	//#define IR_CUSTOM_CODE    0x4EB3					//for NEC Weltrend
	#define IR_KEY_POWER			0xE2						//for NEC
	
#elif(HW_IR_DATA_TYPE == RC5_TYPE)
	#define HW_IR_SEDG              IR_BOTH_EDGE  
  #define HW_IR_EDGE_TYPE         IR_EDGE_FALLING 
  #define HW_IR_OV_INT            OFF
	#define HW_IR_PRE_SCAL          IR_PRE_SCAL_32us
 	#define IR_Count 					RC5_TYPE1
	#define IR_CUSTOM_CODE    0x00FF					//for RC5 Weltrend
	#define IR_KEY_POWER			0x0C						//for RC5
#elif(HW_IR_DATA_TYPE == RC6_TYPE)
  #define HW_IR_SEDG              IR_BOTH_EDGE  
  #define HW_IR_EDGE_TYPE         IR_EDGE_FALLING 
  #define HW_IR_OV_INT            OFF
	#define HW_IR_PRE_SCAL          IR_PRE_SCAL_32us
	#define IR_Count 					RC6_TYPE1
  #define IR_CUSTOM_CODE          0x04FB			//for Weltrend RC6 IR
	#define IR_KEY_POWER							0x0C
#elif(HW_IR_DATA_TYPE == SONY_TYPE)
	#define HW_IR_SEDG              IR_SINGLE_EDGE
	#define HW_IR_EDGE_TYPE         IR_EDGE_FALLING	
	#define HW_IR_PRE_SCAL          IR_PRE_SCAL_32us
	#define HW_IR_OV_INT            ON
 	#define IR_Count 					SONY_TYPE1	
	#define IR_CUSTOM_CODE    0x0AF5					// for SONY
	#define IR_KEY_POWER			0x90						//for SONY

#endif















//For WT61P7
#define IR_PRE_SCAL_1us			0x00
#define IR_PRE_SCAL_8us			0x01
#define IR_PRE_SCAL_32us		0x02
#define IR_PRE_SCAL_64us		0x03
#define IR_PRE_SCAL_128us		0x04
#define IR_PRE_SCAL_256us		0x05
#define IR_PRE_SCAL_512us		0x06
#define IR_PRE_SCAL_1024us		0x07


		enum
		{
		  //H/W IR receive status(for NEC type)
		  IR_IDLE = 0,
		  IR_DATA_BIT0,
		  IR_DATA_BIT1,
		  IR_DATA_BIT2,
		  IR_DATA_BIT3,
		  IR_DATA_BIT4,
		  IR_DATA_BIT5,
		  IR_DATA_BIT6,
		  IR_DATA_BIT7,
		  IR_DATA_BIT8,
		  IR_DATA_BIT9,
		  IR_DATA_BIT10,
		  IR_DATA_BIT11,
		  IR_DATA_BIT12,
		  IR_DATA_BIT13,
		  IR_DATA_BIT14,
		  IR_DATA_BIT15,
		  IR_DATA_BIT16,
		  IR_DATA_BIT17,
		  IR_DATA_BIT18,
		  IR_DATA_BIT19,
		  IR_DATA_BIT20,
		  IR_DATA_BIT21,
		  IR_DATA_BIT22,
		  IR_DATA_BIT23,
		  IR_DATA_BIT24,
		  IR_DATA_BIT25,
		  IR_DATA_BIT26,
		  IR_DATA_BIT27,
		  IR_DATA_BIT28,
		  IR_DATA_BIT29,
		  IR_DATA_BIT30,
		  IR_DATA_BIT31
		};

	//IR NEC data format
#if (HW_IR_PRE_SCAL == IR_PRE_SCAL_64us)
	
	#if(HW_IR_DATA_TYPE == NEC_TYPE)

	#define IR_LEAD_CNT       (BYTE)((13.5*1000*IR_CLK_RATIO)/64)			//210
	#define LEAD_TOLERANCE 		10																			//tolerance=10 Range220~200
	#define IR_REP_CNT        (BYTE)((11.25*1000*IR_CLK_RATIO)/64)		//175
	#define REP_TOLERANCE 		20																			//tolerance=20 Range195~155	

	#define IR_DATA0_CNT      (BYTE)((1.125*1000*IR_CLK_RATIO)/64)              //18
  #define DATA0_TOLERANCE   (BYTE)((1.125*1000*IR_CLK_RATIO*0.2)/64)+1        //tolerance=5 Range23~13    
  #define IR_DATA1_CNT      (BYTE)((2.25*1000*IR_CLK_RATIO)/64)             	//35
  #define DATA1_TOLERANCE   (BYTE)((2.25*1000*IR_CLK_RATIO*0.2)/64)+1         //tolerance=8 Range43~27
	
  #elif(HW_IR_DATA_TYPE == RC5_TYPE)
	/RC5 format (1 clock duty = 1.688mS = 1688uS H->L) 
			#define IR_1PERIOD_CNT        (BYTE)((1.688*1000*IR_CLK_RATIO)/64)/2				
			#define IR_1PERIOD_TOLERANCE 	3	//(BYTE)((1.688*1000*IR_CLK_RATIO*0.2)/64)/2		 
	
	  
   #elif(HW_IR_DATA_TYPE == RC6_TYPE)
	//RC6 format (1 clock duty  = 888uS H->L)
			#define IR_1PERIOD_CNT       14/2 //(BYTE)((0.44*1000*IR_CLK_RATIO)/64)	 
			#define IR_2PERIOD_CNT       28/2// IR_1PERIOD_CNT*2 
			#define IR_1PERIOD_TOLERANCE 5/2 //2*2
			#define IR_2PERIOD_TOLERANCE 6/2 //IR_1PERIOD_TOLERANCE*2
			#define IR_LEAD_CNT         (BYTE)((2.666*1000*IR_CLK_RATIO)/64)	//41
			#define LEAD_TOLERANCE 			 12/2  
										
			#define IR_TR_CNT          	(BYTE)((1.32*1000*IR_CLK_RATIO)/64)	//21
			#define TR_TOLERANCE 		3	
	#endif


#elif (HW_IR_PRE_SCAL == IR_PRE_SCAL_32us)

    #if(HW_IR_DATA_TYPE == RC5_TYPE)
	//RC5 format (1 clock duty = 1.688mS = 1688uS H->L) 
			#define IR_1PERIOD_CNT        (BYTE)((1.688*1000*IR_CLK_RATIO)/32)/2				
			#define IR_1PERIOD_TOLERANCE 3*2
	  
   #elif(HW_IR_DATA_TYPE == RC6_TYPE)
	//RC6 format (1 clock duty  = 888uS H->L) 
			#define IR_1PERIOD_CNT       14 //(BYTE)((0.44*1000*IR_CLK_RATIO)/32)	 
			#define IR_2PERIOD_CNT       28// IR_1PERIOD_CNT*2 
			#define IR_1PERIOD_TOLERANCE 5 //2*2
			#define IR_2PERIOD_TOLERANCE 6 //IR_1PERIOD_TOLERANCE*2
			#define IR_LEAD_CNT         (BYTE)((2.666*1000*IR_CLK_RATIO)/32)	//83
			#define LEAD_TOLERANCE 			 12  
										
			#define IR_TR_CNT          	(BYTE)((1.32*1000*IR_CLK_RATIO)/32)	//41
			#define TR_TOLERANCE 		4*2											//8

		#elif(HW_IR_DATA_TYPE == SONY_TYPE)
 
			#define IR_LEAD_CNT       (BYTE)((3*1000*IR_CLK_RATIO)/32) 				//93
			#define LEAD_TOLERANCE 		(BYTE)((3*1000*IR_CLK_RATIO)*0.2/32)		//18	
			#define IR_DATA0_CNT      (BYTE)((1.2*1000*IR_CLK_RATIO)/32) 			//37.5	
			#define IR_DATA1_CNT      (BYTE)((1.8*1000*IR_CLK_RATIO)/32) 			//56.25
			#define DATA0_TOLERANCE 	(BYTE)((1.2*1000*IR_CLK_RATIO)*0.2/32) //8	
			#define DATA1_TOLERANCE 	(BYTE)((1.8*1000*IR_CLK_RATIO)*0.2/32) //12 	
		
  	#endif		



#elif (HW_IR_PRE_SCAL == IR_PRE_SCAL_128us)
	//For NEC
	#if(HW_IR_DATA_TYPE == NEC_TYPE)
	#define IR_LEAD_CNT       (BYTE)((13.5*1000*IR_CLK_RATIO)/128)			//210
	#define LEAD_TOLERANCE 		10																			//tolerance=10 Range220~200
	#define IR_REP_CNT        (BYTE)((11.25*1000*IR_CLK_RATIO)/128)		//175
	#define REP_TOLERANCE 		20																			//tolerance=20 Range195~155	

	#define IR_DATA0_CNT      (BYTE)((1.125*1000*IR_CLK_RATIO)/128)              //18
  #define DATA0_TOLERANCE   (BYTE)((1.125*1000*IR_CLK_RATIO*0.2)/128)+1        //tolerance=5 Range23~13    
  #define IR_DATA1_CNT      (BYTE)((2.25*1000*IR_CLK_RATIO)/64)             	//35
  #define DATA1_TOLERANCE   (BYTE)((2.25*1000*IR_CLK_RATIO*0.2)/128)+1         //tolerance=8 Range43~27
	#endif

#endif	





















	

	void  DRV_IrInitial(void);
	void  IR_routine(void); 	
	void  Ir_IdelTime(void);
	void  IR_PWR_DOWN_EN_CHK(void);
	void  IR_WAKE_UP_EN_CHK(void);

#if(HW_IR_DATA_TYPE == NEC_TYPE)
  void  HW_IR_NEC_TYPE_routine(void);     
#elif(HW_IR_DATA_TYPE == RC5_TYPE)
  void  HW_IR_RC5_TYPE_routine(void);   
#elif(HW_IR_DATA_TYPE == RC6_TYPE)
  void  HW_IR_RC6_TYPE_routine(void);   
#elif(HW_IR_DATA_TYPE == SONY_TYPE)
  void  HW_IR_SONY_TYPE_routine(void);   
#endif 

#if (HW_IR_DATA_TYPE == NEC_TYPE)   
 #define HW_IR_ISR()       HW_IR_NEC_TYPE_routine(); 
#elif(HW_IR_DATA_TYPE == RC5_TYPE)   
  #define HW_IR_ISR()       HW_IR_RC5_TYPE_routine(); 
#elif(HW_IR_DATA_TYPE == RC6_TYPE)   
  #define HW_IR_ISR()       HW_IR_RC6_TYPE_routine();   
#elif(HW_IR_DATA_TYPE == SONY_TYPE)   
  #define HW_IR_ISR()       HW_IR_SONY_TYPE_routine();   
#endif




	
#endif
